package com.widget.imagebanner;

import android.content.Context;
import android.content.res.TypedArray;
import android.graphics.Bitmap;
import android.media.MediaPlayer;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentActivity;
import androidx.fragment.app.FragmentManager;
import androidx.lifecycle.Lifecycle;
import androidx.viewpager2.adapter.FragmentStateAdapter;
import androidx.viewpager2.widget.ViewPager2;

import android.util.AttributeSet;
import android.util.LayoutDirection;
import android.view.LayoutInflater;
import android.view.MotionEvent;
import android.view.View;
import android.widget.FrameLayout;

import com.widget.imagebanner.view.ImageVideoFragment;
import com.widget.imagebanner.bean.BannerBean;
import com.widget.imagebanner.utils.BitmapUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Timer;
import java.util.TimerTask;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;
import java.util.concurrent.ScheduledExecutorService;

import wseemann.media.FFmpegMediaMetadataRetriever;


public class ImageVideoBanner extends FrameLayout implements ImageVideoFragment.OnVideoCompletionListener {
    private static final int UPTATE_VIEWPAGER = 0;
    private ViewPager2 mViewPager;
    private List<BannerBean> mList = new ArrayList<>();
    private ViewsPagerAdapter mAdapter;
    private int autoCurrIndex = 0;//设置当前 第几个图片 被选中
    private Timer timer;
    private TimerTask timerTask;
    private long period;//轮播图展示时长,默认5秒
    public static ScheduledExecutorService sExecutor;
    public Context mContext;

    public ImageVideoBanner(@NonNull Context context) {
        super(context);
        mContext = context;
        initView(context);
    }

    public ImageVideoBanner(@NonNull Context context, @Nullable AttributeSet attrs) {
        super(context, attrs);
        mContext = context;
        TypedArray typedArray = context.obtainStyledAttributes(attrs, R.styleable.banner);
        period = typedArray.getInt(R.styleable.banner_period, 5000);
        typedArray.recycle();
        initView(context);

    }

    private void initView(Context context) {
        View view = LayoutInflater.from(context).inflate(R.layout.banner_imge_video, this, true);
        mViewPager = view.findViewById(R.id.view_pager);
        mAdapter = new ViewsPagerAdapter(((FragmentActivity) context).getSupportFragmentManager(), ((FragmentActivity) context).getLifecycle());
        mViewPager.setAdapter(mAdapter);
        //ViewPager手势滑动禁用
        mViewPager.setOnTouchListener(new OnTouchListener() {
            @Override
            public boolean onTouch(View v, MotionEvent event) {
                return true;
            }
        });
        mViewPager.registerOnPageChangeCallback(callback);


    }

    public void replaceData(List<com.widget.imagebanner.bean.BannerBean> listBean) {
        mAdapter.replaceData(listBean);
    }

    public void addData(List<com.widget.imagebanner.bean.BannerBean> listBean) {
        mAdapter.addData(listBean);
    }

    /**
     * 初始化视频的Bitmap
     */
    private void initBannerBitmap() {
        if (mList.size() == 0) {
            return;
        }
        for (BannerBean bannerBean : mList) {
            if (bannerBean.getType() == 1 && null == bannerBean.getLoadingImage()) {
                setLoadingImages(bannerBean);
            }
        }
    }


    /**
     * 获取视频的第一帧作为截图
     *
     * @param bannerBean
     */
    private void setLoadingImages(final BannerBean bannerBean) {
        if (sExecutor == null) {
            sExecutor = Executors.newScheduledThreadPool(5);
        }
        Future<Boolean> submit = sExecutor.submit(new Callable<Boolean>() {
            @Override
            public Boolean call() {
                try {
                    String url = bannerBean.getUrl();
                    if (!url.contains("android.resource")) {
                        FFmpegMediaMetadataRetriever mmr = new FFmpegMediaMetadataRetriever();
                        mmr.setDataSource(url);
                        mmr.extractMetadata(FFmpegMediaMetadataRetriever.METADATA_KEY_ALBUM);
                        mmr.extractMetadata(FFmpegMediaMetadataRetriever.METADATA_KEY_ARTIST);
                        Bitmap b = mmr.getFrameAtTime(); // frame at 2 seconds
                        bannerBean.setLoadingImage(BitmapUtils.getBytes(b));
                        mmr.release();
                    }
                    return true;
                } catch (Exception e) {
                    return false;
                }
            }
        });
        try {
            submit.get();
        } catch (InterruptedException e) {
            e.printStackTrace();
        } catch (ExecutionException e) {
            e.printStackTrace();
        }
    }

    private ViewPager2.OnPageChangeCallback callback = new ViewPager2.OnPageChangeCallback() {

        @Override
        public void onPageScrolled(int position, float positionOffset, int positionOffsetPixels) {

        }

        @Override
        public void onPageSelected(int position) {
            autoCurrIndex = position;
//        Toast.makeText(context,"position="+position,Toast.LENGTH_SHORT).show();
            if (mList.get(position).getType() == 1) {
                //如果是视频
                stopBanner();
            }
        }

        @Override
        public void onPageScrollStateChanged(int state) {

        }
    };


    //定时轮播图片，需要在主线程里面修改 UI
    private Handler mHandler = new Handler() {
        public void handleMessage(Message msg) {
            switch (msg.what) {
                case UPTATE_VIEWPAGER:
                    if (msg.arg1 != 0) {
                        mViewPager.setCurrentItem(msg.arg1);
                    } else {
                        //false 当从末页调到首页时，不显示翻页动画效果，
                        mViewPager.setCurrentItem(msg.arg1, false);
                    }
                    break;
            }
        }
    };

    public void stopBanner() {
        if (timer != null) {
            timer.cancel();
            timer = null;
            if (timerTask != null) {
                timerTask.cancel();
                timerTask = null;
            }
        }
    }

    /**
     * 设置轮播图 图片展示间隔
     */
    public void setPeriod(int period) {
        this.period = period;
    }

    public void startBanner() {
        //如果第一页是视频  不用定时器
        if (mList.get(0).getType() == 0) {
            startBanner(period);
        }
    }

    public void startBanner(long delay) {
        stopBanner();
        timer = new Timer();
        createTimerTask();//创建定时器
        timer.schedule(timerTask, delay, period);
    }

    public void createTimerTask() {
        timerTask = new TimerTask() {
            @Override
            public void run() {
                Message message = new Message();
                message.what = UPTATE_VIEWPAGER;
                if (autoCurrIndex == mList.size() - 1) {
                    autoCurrIndex = -1;
                }
                message.arg1 = autoCurrIndex + 1;
                mHandler.sendMessage(message);
            }
        };
    }

    @Override
    public void onVideoCompletion(MediaPlayer mp) {
        if (mList.size() == 1) {
            //当集合中只有一个视频时，循环播放
            mAdapter.getFragment().circulationPlayer();
        }
        startBanner(0);
    }

    @Override
    public void onError(MediaPlayer mp) {
        startBanner(0);
    }

    private final class ViewsPagerAdapter extends FragmentStateAdapter {
        private ImageVideoFragment fragment;
        private FragmentManager fm;

        public ViewsPagerAdapter(@NonNull FragmentManager fragmentManager, @NonNull Lifecycle lifecycle) {
            super(fragmentManager, lifecycle);
            this.fm = fragmentManager;
        }


        @NonNull
        @Override
        public Fragment createFragment(int position) {
            fragment = new ImageVideoFragment();
            fragment.setOnVideoCompletionListener(ImageVideoBanner.this);
            com.widget.imagebanner.bean.BannerBean bannerBean = mList.get(position);
            Bundle bundle = new Bundle();
            bundle.putSerializable("bannerBean", bannerBean);
            bundle.putBoolean("loop", getLoop());
            fragment.setArguments(bundle);
            return fragment;
        }

        @Override
        public int getItemCount() {
            return mList.size();
        }

        /**
         * 只有1个视频的时候循环播放
         *
         * @return
         */
        private boolean getLoop() {
            if (mList.size() == 1 && mList.get(0).getType() == 1) {
                return true;
            }
            return false;
        }

        public void replaceData(List<BannerBean> listBean) {
            // 新数据和原来数据对比，不一致才去刷新
            boolean change = compareData(listBean);
            if (!change) {
                notifyDataSetChanged();
//                startBanner();
                return;
            }
            if (null != listBean) {
                mList.clear();
                addData(listBean);
                //如果第一张是图片 需要开启定时器
                if (listBean.get(0).getType() == 0) {
                    startBanner();
                }
                autoCurrIndex = 0;
//                startBanner();
            }
        }


        /**
         * 对比数据
         *
         * @param listBean
         * @return
         */
        private boolean compareData(List<BannerBean> listBean) {
            if (listBean == null || listBean.size() == 0) {
                return false;
            }
            if (listBean.size() == mList.size()) {
                for (int i = 0; i < mList.size(); i++) {
                    if (mList.get(i).getType() != listBean.get(i).getType()
                            || !mList.get(i).getUrl().equals(listBean.get(i).getUrl())) {
                        return true;
                    }
                }
                return false;
            }
            return true;
        }


        public void addData(List<BannerBean> listBean) {
            if (null != listBean) {
                mList.addAll(listBean);
            }
            initBannerBitmap();
            notifyDataSetChanged();
        }

        public ImageVideoFragment getFragment() {
            return fragment;
        }


    }
}
